<?php
namespace Geral\Logic\ParceiroNegocio;

use \Geral\Util\NovoCodigo;
use \Geral\Util\Ambiente;
use \Geral\Entities\Parcnegocio  as ParcnegocioEntity;
use \Geral\Entities\Enderecoparc as EnderecoparcEntity;
use \Geral\Entities\Tipoestrutura;
use \Geral\Entities\Parceria;
use \Geral\Entities\Fornecedor;
use \Geral\Entities\Transport;
use \Geral\Entities\Cliente;
use \Geral\Entities\Operador;
use \Geral\Entities\Funcionario;
use Indeterminado\Entities\Relacionaparc;
use Geral\Entities\Vendedor;
use Geral\Logic\ParceiroNegocio\Enderecoparc;
use \Geral\Entities\Prospect;

/**
 * Parcnegociologic.
 *
 * @package Logic
 * @author  erikasouza
 *
 */
class Parcnegocio {

    /**
    * ################################## REGRA DE UTILIZACAO DA CLASSE #####################################
    *
    * Ao criar ou modificar um metodo, inserir phpdoc contendo descricoes do metodo, parametros e retorno.
    * Descrever o autor e data de criacao ou alteracao.
    * Verificar metodo insereParceiroPessoa nesta classe.
    *
    * Verificar identacao, nomeclatura padrao Code Sniffer.
    *
    * ######################################################################################################
    */

    const CDTIPOINSCRICAO_CPF = 'CPF';
    const CDTIPOINSCRICAO_CNPJ = 'CNPJ';
    const CDTIPOINSCRICAO_LIVRE = 'LIVRE';
    const CDTIPOPARCPRINCIPAL_PESSOA = 'PESSOA';
    const CDTIPOPARCPRINCIPAL_ESTRUTURA = 'ESTRUTURA';
    const CDTIPOPARCPRINCIPAL_FUNCIONARIO = 'FUNCIONARIO';
    const TIPOESTRUTURA_EMPRESA = 1;
    const TIPOESTRUTURA_FILIAL = 2;
    const NMTIPOESTRUTURA_EMPRESA = "EMPRESA";
    const NMTIPOESTRUTURA_FILIAL = "FILIAL";

    /**
     * retorna um sequencial para NRPARCNEGOCIO.
     *
     * @return number
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function geraNovoCodigoParcnegocio(){
        return NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'PARCNEGOCIO');
    }

    /**
     * retorna um sequencial para NRCOMUNICAPARC.
     *
     * @return number
     *
     * @author marcelosilva - 05/06/2013
     */
    public static function geraNovoCodigoComunicaparc(){
        return Novocodigo::geraCodigo(Ambiente::getNrorg(), 'COMUNICAPARC');
    }

    /**
     * retorna um sequencial para Nrinscricaoparc.
     *
     * @return number
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function geraNovoCodigoNrInscr(){
        return Novocodigo::geraCodigo(Ambiente::getNrorg(), 'VENDEDOR.NRINS', 1, 5);
    }
    
     /**
     * retorna do banco o numero de inscrição do parceiro de negócio.
     *
     * @param integer $nrparcnegocio    Numero do Parceiro de Negócio
     * @param integer $nrorg            Numero da Organização
     *
     * @return number
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function selectNrinscricaoParc($nrparcnegocio, $nrorg){
        return \Geral\Entities\Parcnegocio::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
    }
    
     /**
     * retorna do banco o tamanho do numero de inscrição.
     *
     * @param integer $nrparcnegocio    Numero do Parceiro de Negócio
     * @param integer $nrorg            Numero da Organização
     *
     * @return number
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function lenghNrInscricaoParc($nrparcnegocio, $nrorg){
        $parcnegocio = ParcnegocioEntity::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
        return strlen($parcnegocio->getNrinscricaoparc());
    }

    /**
     * deleta os filhos de parceiro de negócio para poder excluir o registro.
     *
     * @param integer $nrparcnegocio     Numero do Parceiro de Negócio
     * @param integer $nrorg            Numero da Organização
     * @return boolean
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function deleteChilds($nrparcnegocio, $nrorg){
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        try{
            $tipos = tk_get_field_value('TIPODEPARCEIRO');
            $array = explode(',', $tipos);

            if(in_array('TRANSPORTADORA',$array)){
                $deletetipo = "delete from transport where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }

            foreach ($array as $tipo){
                if(!empty($tipo)){
                    if($tipo === 'INTERLOCUTOR'){
                        $nrrelacionaparcselect = "select nrrelacionaparc from relacionaparc where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                        $nrrelacionaparcarray = \Setforms2\Runtime\Connection::getInstance()->fetchAll($nrrelacionaparcselect);
                        if(empty($nrrelacionaparcarray)){
                            $nrrelacionaparc = '\'\'';
                        }else{
                            $nrrelacionaparc = $nrrelacionaparcarray[0]['NRRELACIONAPARC'];
                        }

                        $deleteinteclie = "delete from INTECLIE where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                        \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteclie);

                        $deleteinteforn = "delete from INTEFORN where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                        \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteforn);

                        $deleteinteprosp = "delete from INTEPROSP where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                        \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteprosp);

                        $deleterelacionaparcnegrelac = "delete from RELACIONAPARC where nrparcnegrelac = ".$nrparcnegocio." and nrorg = ".$nrorg;
                        \Setforms2\Runtime\Connection::getInstance()->execute($deleterelacionaparcnegrelac);
                    }
                    else if($tipo != 'TRANSPORTADORA'){
                        $deletetipo = "delete from ".$tipo." where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                        \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
                    }
                }
            }

            $deleterelacionaparc = "delete from RELACIONAPARC where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
            \Setforms2\Runtime\Connection::getInstance()->execute($deleterelacionaparc);

            $deleteparceria = "delete from parceria where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
            \Setforms2\Runtime\Connection::getInstance()->execute($deleteparceria);

            // delete enderecoparc e seus filhos
            $nrenderecoparc = "select nrenderecoparc from enderecoparc where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
            $array_nrenderecoparc = \Setforms2\Runtime\Connection::getInstance()->fetchAll($nrenderecoparc);

            $deletecomunicaparc = "delete from COMUNICAPARC where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
            \Setforms2\Runtime\Connection::getInstance()->execute($deletecomunicaparc);

            foreach($array_nrenderecoparc as $nrende){
                $deleteendeclie = "delete from ENDECLIE where nrenderecoparc = ".$nrende['NRENDERECOPARC']." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteendeclie);

                $deleteendeforn = "delete from ENDEFORN where nrenderecoparc = ".$nrende['NRENDERECOPARC']." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteendeforn);

                $deleteendeprosp = "delete from ENDEPROSPECT where nrenderecoparc = ".$nrende['NRENDERECOPARC']." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteendeprosp);
            }

            $deleteendereco = "delete from enderecoparc where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
            \Setforms2\Runtime\Connection::getInstance()->execute($deleteendereco);

            $db->getConn()->commit();
        }catch (\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }
    
     /**
     * deleta parceria do parceiro de negócio de acordo com a where clause.
     *
     * @param integer $nrparcnegocio    Numero do Parceiro de Negócio
     * @param integer $where            Where Clause para exclusão
     *
     * @return void
     *
     * @author gustavosilva - 17/05/2012
     */
    public static function deleteParceriaByWhere($nrparcnegocio,$where)
    {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        try{
            $auxwhere = explode(',', $where);
            $inwhere = '\''.implode('\',\'', $auxwhere).'\'';
            $array = explode(',', $where);
            $nrorg = Ambiente::getNrorg();
            if(in_array('INTERLOCUTOR', $array)=== false){
                $nrrelacionaparcselect = "select nrrelacionaparc from relacionaparc where nrparcnegrelac = ".$nrparcnegocio." and nrorg = ".$nrorg;
                $nrrelacionaparcarray = \Setforms2\Runtime\Connection::getInstance()->fetchAll($nrrelacionaparcselect);
                if(!empty($nrrelacionaparcarray)){
                    $nrrelacionaparc = $nrrelacionaparcarray[0]['NRRELACIONAPARC'];
                }else{
                    $nrrelacionaparc = '\'\'';
                }

                $deleteinteclie = "delete from INTECLIE where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteclie);

                $deleteinteforn = "delete from INTEFORN where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteforn);

                $deleteinteprosp = "delete from INTEPROSP where nrrelacionaparc = ".$nrrelacionaparc." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deleteinteprosp);

                $deleterelacionaparc = "delete from RELACIONAPARC where nrparcnegrelac = ".$nrparcnegocio." and nrorg = ".$nrorg;
                $db = \Setforms2\Runtime\Connection::getInstance();
                $db->execute($deleterelacionaparc);
            }

            if(in_array('CLIENTE', $array)=== false){
                $cdcliente = "select cdcliente from cliente where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                $var = \Setforms2\Runtime\Connection::getInstance()->fetchAll($cdcliente);
                foreach ($var as $cdcliente){
                    $cd = '\''.$cdcliente['CDCLIENTE'].'\'';
                    $endeclie = "delete from endeclie where cdcliente = ".$cd." and nrorg = ".$nrorg;
                    \Setforms2\Runtime\Connection::getInstance()->execute($endeclie);
                }

                $deletetipo = "delete from CLIENTE where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }

            if(in_array('FORNECEDOR', $array)=== false){
                $cdfornecedor = "select cdforneced from fornecedor where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                $var = \Setforms2\Runtime\Connection::getInstance()->fetchAll($cdfornecedor);
                $transport = \Geral\Entities\Transport::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                $transport->delete();
                foreach ($var as $cdforn){
                    $cd = '\''.$cdforn['CDFORNECED'].'\'';
                    $endeforn = "delete from endeforn where cdforneced = ".$cd." and nrorg = ".$nrorg;
                    \Setforms2\Runtime\Connection::getInstance()->execute($endeforn);
                }

                // deleta transportadora filha antes do fornecedor
                $deletetipo = "delete from FORNECEDOR where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }
            if(in_array('FUNCIONARIO', $array)=== false){
                $deletetipo = "delete from FUNCIONARIO where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }
            if(in_array('VENDEDOR', $array)=== false){
                $deletetipo = "delete from VENDEDOR where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }
            if(in_array('OPERADOR', $array)=== false){
                $deletetipo = "delete from OPERADOR where nrparcnegocio = ".$nrparcnegocio." and nrorg = ".$nrorg;
                \Setforms2\Runtime\Connection::getInstance()->execute($deletetipo);
            }

            $delete = "delete from parceria where nrparcnegocio = '$nrparcnegocio' and cdtipoparceiro not in ({$inwhere}) and nrorg = ".$nrorg;
            \Setforms2\Runtime\Connection::getInstance()->execute($delete);

            $db->getConn()->commit();

        }catch(\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }


    /**
     * Retorna os registros de um determinado parceiro de neg?.
     *
     * @param integer $nrparceiro N?o do parceiro de neg?.
     * @param integer $nrorg      N?o da Organiza?.
     *
     * @return \Geral\Entities\Parcnegocio
     */
    public function retornaParceiroNegocio($nrparceiro, $nrorg){
        try {
            return \Geral\Entities\Parcnegocio::findOneByNrparcnegocioAndNrorg($nrparceiro, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os registros de um determinado parceiro de negócio por inscrição.
     * @param $cdtipoinscricao
     * @param integer $nrinscricaoparc       Nr. do parceiro de negócio.
     * @param integer $nrorg                 Nr. da Organização.
     * @return \Geral\Entities\Enderecoparc
     * @return \Geral\Entities\Enderecoparc  Parceiro por inscricao.
     */
    public function retornaParceiroPorInscricao($cdtipoinscricao, $nrinscricaoparc, $nrorg){
        try {
            return \Geral\Entities\Parcnegocio::findOneByCdtipoinscricaoAndNrinscricaoparcAndNrorg($cdtipoinscricao, $nrinscricaoparc, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function manipulaParceiro($nrorg, $nrinscricaoparc, $nmprincipalparc, $nmsecundariparc, $cdtipoinscricao, $idativo, $idpessoafisica, $idinstituicao) {
        try {
            /** @var $parceiro ParcnegocioEntity */
            $parceiro = $this->retornaParceiroPorInscricao($cdtipoinscricao, $nrinscricaoparc, $nrorg);
            if (!($parceiro instanceof ParcnegocioEntity)){
                $dtinclusao = \Geral\Util\DateUtil::getDataAtual()->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $parceiro = new ParcnegocioEntity();
                $parceiro->setNrorg($nrorg);
                $parceiro->setNmprincipalparc($nmprincipalparc);
                $parceiro->setNmsecundariparc($nmsecundariparc);
                $parceiro->setNrinscricaoparc($nrinscricaoparc);
                $parceiro->setCdtipoinscricao($cdtipoinscricao);
                $parceiro->setIdativo($idativo);
                $parceiro->setDtinclusao($dtinclusao);
                $parceiro->setIdpessoafisica($idpessoafisica);
                $parceiro->setIdinstituicao($idinstituicao);
                $parceiro->persist();
                $connection->flush();
            }
            return $parceiro;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere um parceiro de negocio e uma parceria para a pessoa.
     *
     * @param integer $nrOrg           Numero da organizacao.
     * @param string  $nrInscricaoParc Numero de inscricao do parceiro.
     * @param string  $nmPrincipalParc Nome principal do parceiro.
     * @param string  $nmSecundariParc Nome secundario do parceiro.
     * @param string  $dtNascPessoa    Data de nascimento do parceiro.
     *
     * @return Parcnegocio
     *
     * @author tuliobraga/brunofeliciano
     *         thalesvalias - 19/04/2012 - insere na tabela de parceria tambem e outros ajustes
     */
    public function insereParceiroPessoa($nrOrg, $nrInscricaoParc, $nmPrincipalParc, $nmSecundariParc, $dtNascPessoa) {

        if (empty($nrInscricaoParc) === true || strlen($nrInscricaoParc) < 11){
            $tipoInscricao = self::CDTIPOINSCRICAO_LIVRE;
        } else {
            $tipoInscricao = self::CDTIPOINSCRICAO_CPF;
        }
        $parceiro = $this->retornaParceiroPorInscricao($tipoInscricao, $nrInscricaoParc, $nrOrg);
        if (!($parceiro instanceof ParcnegocioEntity)){
            $dtNascPessoa = \Geral\Util\DateUtil::getDataDeString($dtNascPessoa);
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            try {
                $nrparcnegocio = \Windows\SfGerParcnegocio\Ger0110001\Facade::geraNovoCodigoParcnegocio();
                $parceiro = new ParcnegocioEntity;
                $parceiro->setNrparcnegocio($nrparcnegocio);
                $parceiro->setNrorg($nrOrg);
                $parceiro->setNmprincipalparc($nmPrincipalParc);
                $parceiro->setNmsecundariparc($nmSecundariParc);
                $parceiro->setNrinscricaoparc($nrInscricaoParc);
                $parceiro->setCdtipoinscricao($tipoInscricao);
                $parceiro->setIdparcfundido('N');
                $parceiro->setIdativo(ParcnegocioEntity::IDATIVO_SIM);
                $parceiro->setIdpessoafisica(ParcnegocioEntity::IDPESSOAFISICA_SIM);
                $parceiro->setIdinstituicao(ParcnegocioEntity::IDINSTITUICAO_NAO);
                $parceiro->setCdtipoparcprincipal(ParcnegocioEntity::CDTIPOPARCPRINCIPAL_PESSOA);
                $parceiro->setDtnascifundparc($dtNascPessoa);
                $parceiro->persist();
                $connection->flush();
            } catch (\Exception $e) {
                throw $e;
            }
            return $parceiro;
        }

        $nrParceiroNegocio = $parceiro->getNrparcnegocio();

        $this->_insereParceria(
            $nrOrg, $nrParceiroNegocio, ParcnegocioEntity::CDTIPOPARCPRINCIPAL_PESSOA, ParcnegocioEntity::IDATIVO_SIM);
        return $parceiro;

    }


    /**
     * Insere um parceiro de negocio e uma parceria para a estrutura empresa ou filial.
     *
     * @param integer $nrOrg           Numero da organizacao.
     * @param string  $nrInscricaoParc Numero de inscricao do parceiro.
     * @param string  $nmPrincipalParc Nome principal do parceiro.
     * @param string  $nmSecundariParc Nome secundario do parceiro.
     * @param integer $nrTipoEstrutura   Tipo da estrutura.
     *
     * @return ParcnegocioEntity
     *
     * @author thalesvalias - 25/04/2012
     */
    public function insereParceiroEstrutura($nrOrg, $nrInscricaoParc, $nmPrincipalParc, $nmSecundariParc, $nrTipoEstrutura) {
        //retorna o tipo de estrutura
        $tipoEstrutura = Tipoestrutura::findOneByNrtipoestruturaAndNrorg($nrTipoEstrutura, $nrOrg);
        $nomeTipoEstrutura = $tipoEstrutura->getNmtipoestrutura();
        $nrParceiroNegocio = null;
        if ($nomeTipoEstrutura === self::NMTIPOESTRUTURA_EMPRESA || $nomeTipoEstrutura === self::NMTIPOESTRUTURA_FILIAL) {
            $parceiro = $this->retornaParceiroPorInscricao(self::CDTIPOINSCRICAO_CNPJ, $nrInscricaoParc, $nrOrg);
            if ($parceiro === null) {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                try {
                    $parceiro = new ParcnegocioEntity;
                    $parceiro->setNrorg($nrOrg);
                    $parceiro->setNmprincipalparc($nmPrincipalParc);
                    $parceiro->setNmsecundariparc($nmSecundariParc);
                    $parceiro->setNrinscricaoparc($nrInscricaoParc);
                    $parceiro->setCdtipoinscricao(self::CDTIPOINSCRICAO_CNPJ);
                    $parceiro->setIdativo(ParcnegocioEntity::IDATIVO_SIM);
                    $parceiro->setIdpessoafisica(ParcnegocioEntity::IDPESSOAFISICA_NAO);
                    $parceiro->setIdinstituicao(ParcnegocioEntity::IDINSTITUICAO_SIM);
                    $parceiro->setCdtipoparcprincipal(self::CDTIPOPARCPRINCIPAL_ESTRUTURA);
                    $parceiro->persist();
                    $connection->flush();
                } catch (\Exception $e) {
                    throw $e;
                }
            }
            $nrParceiroNegocio = $parceiro->getNrparcnegocio();

            $this->_insereParceria(
                 $nrOrg, $nrParceiroNegocio, self::CDTIPOPARCPRINCIPAL_ESTRUTURA, Parceria::IDATIVO_SIM);
        }
        return $nrParceiroNegocio;
    }


    /**
     * Insere uma parceria para o parceiro de negocio de determinada pessoa.
     * @param $nrOrg             Numero da organizacao.
     * @param $nrParceiroNegocio Numero do parceiro de negocio da pessoa.
     * @param $tipoParceiro      Tipo do Parceiro de Negocio (PESSOA).
     * @param $idAtivo           Identificador de atividade da parceria (S).
     * @return void
     * @author thalesvalias - 19/04/2012
     */
    private function _insereParceria($nrOrg, $nrParceiroNegocio, $tipoParceiro, $idAtivo) {
        $parceria = Parceria::findOneByNrparcnegocioAndCdtipoparceiroAndNrorg($nrParceiroNegocio, $tipoParceiro, $nrOrg);
        if ($parceria === null) {
            try {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $parceria = new Parceria;
                $parceria->setNrorg($nrOrg);
                $parceria->setNrparcnegocio($nrParceiroNegocio);
                $parceria->setCdtipoparceiro($tipoParceiro);
                $parceria->setIdativo($idAtivo);
                $parceria->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());
                $parceria->setCdoperinclusao(Ambiente::getCdOperador());
                $parceria->persist();
                $connection->flush();
            } catch (\Exception $e) {
                throw $e;
            }
        }

    }

    /**
     * Atualiza determinado parceiro de negocio de acordo com alteracoes em pessoa.
     *
     * @param $nrParceiroNegocio Numero do parceiro da pessoa.
     * @param $nrOrg             Numero da organizacao.
     * @param $nomePessoa        Nome da pessoa.
     * @param $nrInscricaoPessoa Numero de inscricao da pessoa.
     * @param $dtNascPessoa      Data de nascimento dda pessoa.
     *
     * @return void
     *
     * @author thalesvalias - 19/04/2012
     *         alessandrobartels - 
     */
    public function atualizaParceiroPessoa($nrParceiroNegocio, $nrOrg, $nomePessoa, $nrInscricaoPessoa, $dtNascPessoa) {
        $parceiroNegocio = ParcnegocioEntity::findOneByNrparcnegocioAndNrorg($nrParceiroNegocio, $nrOrg);
        if ($parceiroNegocio !== null) {
            $nomeParceiro = $parceiroNegocio->getNmprincipalparc();
            $nrInscricaoParceiro = $parceiroNegocio->getNrinscricaoparc();
            $dtNascParceiro = (string)$parceiroNegocio->getDtnascifundparc();

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            if ($nomeParceiro !== $nomePessoa) {
                $parceiroNegocio->setNmprincipalparc($nomePessoa);
                $parceiroNegocio->setNmsecundariparc($nomePessoa);
            }

            if ( ($nrInscricaoParceiro !== $nrInscricaoPessoa) && (empty($nrInscricaoPessoa) === false)) {
                $parceiroNegocio->setNrinscricaoparc($nrInscricaoPessoa);
            }

            if ($dtNascParceiro !== $dtNascPessoa) {
                $parceiroNegocio->setDtnascifundparc($dtNascPessoa);
            }

            try {
                $parceiroNegocio->persist();
                $connection->flush();
            } catch(\Exception $e) {
                tk_message(' Exception '.$e->getLine().' : '.$e->getTrace().' - '.$e->getMessage());
                tk_show_message_by_name('msg_parceiro_negocio_nao_atualizado');
            }
        } else {
            tk_show_message_by_name('msg_parceiro_ncad_pessoa');
        }
    }


    /**
     * Retorna a consulta de Parceiros de negocio com parceria de operador.
     *
     * @return  <$sql, string>
     */
    public static function buscaParceirosOperadores(){
        /** @var $parcnegocioRepository \Geral\Repositories\Parcnegocio */
        $parcnegocioRepository = ParcnegocioEntity::getRepository();
        $dados = $parcnegocioRepository->buscaParceirosOperadores();
        return $dados;
    }

    /**
     * Rotina que prepara o cadastro de um novo parceiro de negócio, nele é validado o Tipo de Parceiro
     * setando o valor de CDTIPOPARCPRINCIPAL com o primeiro Tipo de Parceiro do Multi-CheckedBox.s
     *
     * @param array     tipos   Tipos de Parceiro de negócio
     *
     * @return void
     *
     * @author gustavosilva
     * 17/05/2012
     */
    public static function rotinaDeCadastro($tipos){
        if ($tipos[0] === '')
            throw new \Exception(tk_get_message_by_name('msg_marque_tipo_parceiro'));
        else
            tk_set_field_value('CDTIPOPARCPRINCIPAL', $tipos[0]);
    } 
    
     /**
     * Rotina de criação do registro de transportadora.
     *
     * @param integer $nrparcnegocio    Numero do Parceiro de Negócio
     * @param integer $nrorg            Numero da Organização
     * @param array $transportadora     Array de dados para transportadora
     *
     * @return Geral\Entities\Transport
     *
     * @author gustavosilva
     *         17/05/2012
     */
    
    public static function rotinaTransportadora($inArray, $nrparcnegocio, $transportadora, $nrorg)
    {
        if ($inArray == true){
            $transport = Transport::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
            if (empty($transport)){
                try {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $fornecedor = Fornecedor::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                    $cdforneced = $fornecedor->getCdforneced();

                    $transport = new Transport;
                    $transport->setIdtpijurtran($transportadora['CDTIPOINSCRICAO']);
                    $transport->setCdforneced($cdforneced);
                    $transport->setNrinsjurtran($transportadora['NRINSCRICAOPARC']);
                    $transport->setNmrazsoctran($transportadora['NMINSCRICAOPARC']);
                    $transport->setNmfanttran($transportadora['NMSECUNDARIPARC']);
                    $transport->setNrinsesttran($transportadora['NRINSESTTRAN']);
                    $transport->setDsendetran($transportadora['ENDERECOTRAN']);
                    $transport->setNmbairtran($transportadora['NMBAIRROTRAN']);
                    $transport->setNrceptran($transportadora['NRCEPTRAN']);
                    $transport->setCdpais($transportadora['CDPAIS']);
                    $transport->setSgestado($transportadora['SGESTADO']);
                    $transport->setCdmunicipio($transportadora['CDMUNICIP']);
                    $transport->setNrorg($transportadora['NRORG']);
                    $transport->setIdcomparqxmltra('S');
                    $transport->setNrparcnegocio($nrparcnegocio);
                    $transport->setIdativo('S');
                    $transport->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());
                    $transport->setCdoperinclusao(Ambiente::getCdOperador());
                    $transport->setNrorginclusao(Ambiente::getNrorg());
                    $transport->persist();
                    $connection->flush();
                } catch (\Exception $e) {
                    throw $e;
                }
            }
            else{
                $transportRepository = Transport::getRepository();
                $transportRepository->updateTransportadora($nrparcnegocio, $transportadora, $nrorg);
            }
        }
        else{
            $transport = Transport::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
            if (!empty($transport)){
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $transport->setIdativo('N');
                $transport->persist();
                $connection->flush();
            }
        }
    }

     /**
     * Rotina de criação de parceria do parceiro de negócio.
     *
     *
     * @param array  $tipos             Array de Tipo de Parceiro
     * @param integer    $nrparcnegocio     Numero do parceiro de negócio
     * @param integer    $nrorg         Numero da Organização
     * @param integer    $cdtipoinscricao   Código do tio de Inscrição
     * @param integer    $nrinscricaoparc   Numero da Inscrição do parceiro
     * @param string     $nome          Nome Primário
     * @param string     $nomereduzido      Nome Secundário
     * @param date   $dtnascimento      Data de Nascimento
     * @param integer    $cdempresa     Código da Empresa
     * @param integer    $nrparcnegrelac    Numero do Parceiro de Relacionamento
     *
     * @return void
     *
     * @author gustavosilva
     *         17/05/2012
     */
    public static function rotinaParceria($tipos, $nrparcnegocio, $nrorg, $cdtipoinscricao, $nrinscricaoparc, $nome, $nomereduzido, $dtnascimento, $cdempresa, $nrparcnegrelac, $transportadora, $cdsegmer){

        try{
            
            $tipoinscricao = array('CPF' => 'F', 'CNPJ' => 'J', 'LIVRE' => 'I', 'CODIGO' => 'O');
            
            $var = explode(',', $tipos);
            //se não existir em parceria criar um novo registro             
            foreach ($var as $tipo){
                $obj = Parceria::findOneByNrparcnegocioAndCdtipoparceiroAndNrorg($nrparcnegocio, $tipo, $nrorg);
                if (!$obj instanceof Parceria){
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $obj = new Parceria();
                    $obj->setNrparceria(Novocodigo::geraCodigo($nrorg, 'PARCERIA', 1, 10));
                    $obj->setNrorg($nrorg);
                    $obj->setNrparcnegocio($nrparcnegocio);
                    $obj->setCdtipoparceiro($tipo);
                    $dtinclusao = \Geral\Util\DateUtil::getDataAtual();
                    $obj->setDtinclusao($dtinclusao);
                    $obj->setCdoperinclusao(Ambiente::getCdOperador());
                    $obj->setNrorginclusao(Ambiente::getNrorg());
                    $obj->setIdativo('S');
                    try{
                        $obj->persist();
                        $connection->flush();
                    }catch (\Exception $e){
                        throw  $e;
                    }
                }
                else {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    if (self::isNewItemprebase($obj) === true){
                        $obj->setNrparceria(Novocodigo::geraCodigo(Ambiente::getNrorg(), 'PARCERIA', 1, 10));
                        $obj->setNrparcnegocio($nrparcnegocio);
                        $obj->setCdtipoparceiro($tipo);
                        $obj->setIdativo('S');
                        $obj->setDtultatu(\Geral\Util\DateUtil::getDataAtual());
                        $obj->setCdoperultatu(Ambiente::getCdOperador());
                        $obj->setNrorgultatu(Ambiente::getNrorg());
                        try{
                            $obj->persist();
                            $connection->flush();
                        }catch (\Exception $e){
                            throw  $e;
                        }
                    }
                    else{
                        $obj->setIdativo('S');
                        $obj->setDtultatu(\Geral\Util\DateUtil::getDataAtual());
                        $obj->setCdoperultatu(Ambiente::getCdOperador());
                        $obj->setNrorgultatu(Ambiente::getNrorg());
                        try{
                            $obj->persist();
                            $connection->flush();
                        }
                        catch (\Exception $e){
                            throw  $e;
                        }
                    }
                }
            }

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            //deleta registros existentes que não estão marcados
            $inwhere = implode(',', $var);
            $where = "NRPARCNEGOCIO = ? and (INSTR(?, CDTIPOPARCEIRO) = 0) and NRORG = ?";
            $parceria = Parceria::getObjectsByWhere($where, array($nrparcnegocio, $inwhere, $nrorg));

            /** @var $chave \Geral\Entities\Parceria */
            foreach ($parceria as $chave){
                $chave->setIdativo('N');
                $chave->setDtultatu(\Geral\Util\DateUtil::getDataAtual());
                $chave->setCdoperultatu(Ambiente::getCdOperador());
                $chave->setNrorgultatu(Ambiente::getNrorg());
                $chave->persist();
            }
            if(in_array('CLIENTE', $var)){
                $cliente = Cliente::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if(empty($cliente)){
                    $cliente = Cliente::findOneByNrinsjurclieAndNrorg($nrinscricaoparc, $nrorg);
                    if(empty($cliente)){
                        try{
                            $cliente = new Cliente();
                            $cliente->setNrparcnegocio($nrparcnegocio);
                            $cliente->setCdcliente($tipoinscricao[$cdtipoinscricao].$nrinscricaoparc);
                            $cliente->setIdtpijurclie($tipoinscricao[$cdtipoinscricao]);
                            $cliente->setNrinsjurclie($nrinscricaoparc);
                            $cliente->setNmrazsocclie($nome);
                            $cliente->setNmfantclie($nomereduzido);
                            $cliente->setCdsegmer($cdsegmer);
                            if ($dtnascimento != null){
                                $dtnascimento = \Geral\Util\DateUtil::truncateData(\Geral\Util\DateUtil::getDataDeString($dtnascimento));
                                $cliente->setDtnascclie($dtnascimento);
                            }
                            $cliente->setNrorg($nrorg);
                            $cliente->setIdpripubclie('V');
                            $cliente->setIdversaldcli('N');
                            $cliente->setIdutsaldcli('N');
                            $cliente->setIdcalcdvclie('1');
                            $cliente->setIdutsubclien('N');
                            $cliente->setIdzersaldcon('S');
                            $cliente->setIdgernfclie('1');
                            $cliente->setIdsitfinclie('N');
                            $cliente->setIdligextclie('S');
                            $cliente->setIdtplimatefat('P');
                            $cliente->setNrlimatemes(10);
                            $cliente->setIdlibatdcliinad('N');
                            $cliente->setIdpgfrettrancli('9');
                            $cliente->setIdcomparqxmlcli('S');
                            $cliente->setIdclisecatclpri('N');
                            $cliente->setIdgravobscliped('S');
                            $cliente->setIdexbtpnomeclie('F');
                            $cliente->setIdutdescicms('N');
                            $cliente->setIdativo('S');
                            $cliente->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());
                            $cliente->setCdoperinclusao(Ambiente::getCdOperador());
                            $cliente->setNrorginclusao(Ambiente::getNrorg());
                            $cliente->persist();
                        }catch(\Exception $e){
                            throw $e;
                        }
                    }
                    else{
                        $cliente->setIdativo('S');
                        $cliente->persist();
                    }
                }
                else{
                    $cliente->setIdativo('S');
                    $cliente->persist();
                }
            }else{
                $cliente = Cliente::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if(!empty($cliente)){
                    $cliente->setIdativo('N');
                    $cliente->persist();
                }

            }

            if(in_array('FORNECEDOR', $var)){
                $fornecedor = Fornecedor::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if($fornecedor == null){
                    try{
                        $fornecedor = Fornecedor::findOneByNrinsjurfornAndNrorg($nrinscricaoparc, $nrorg);
                        if($fornecedor == null){
                            $fornecedor = new Fornecedor();
                            $fornecedor->setNrparcnegocio($nrparcnegocio);
                            $fornecedor->setCdforneced($tipoinscricao[$cdtipoinscricao].$nrinscricaoparc);
                            $fornecedor->setIdtpijurforn($tipoinscricao[$cdtipoinscricao]);
                            $fornecedor->setNrinsjurforn($nrinscricaoparc);
                            $fornecedor->setNmrazsocforn($nome);
                            $fornecedor->setNmfantforn($nomereduzido);
                            $fornecedor->setIdpripubforn('V');
                            $fornecedor->setIdagrubolfor('N');
                            $fornecedor->setIdligextforn('N');
                            $fornecedor->setIdutqtdispforn('N');
                            $fornecedor->setIdbloqcmpsaffor('N');
                            $fornecedor->setIdlocalforncad('F');
                            $fornecedor->setIdsexoprofi('I');
                            $fornecedor->setIdprofipossveic('N');
                            $fornecedor->setIdavaliaprofi('0');
                            $fornecedor->setIdnpermancotint('N');
                            $fornecedor->setIdcomparqxmlfor('S');
                            $fornecedor->setIdtipopagaprof('CONTAPAGAR');
                            $fornecedor->setNrorg($nrorg);
                            $fornecedor->setIdativo('S');
                            $fornecedor->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());
                            $fornecedor->setCdoperinclusao(Ambiente::getCdOperador());
                            $fornecedor->setNrorginclusao(Ambiente::getNrorg());
                            $fornecedor->persist();
                        }
                    }catch(\Exception $e){
                        throw $e;
                    }
                }
                else{
                    $fornecedor->setIdativo('S');
                    $fornecedor->persist();
                }
            }else{
                $fornecedor = Fornecedor::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if($fornecedor !== null){
                    $fornecedor->setIdativo('N');
                    $fornecedor->persist();
                }
            }

            if(in_array('INTERLOCUTOR', $var)){
               $interlocutor = Relacionaparc::findOneByNrparcnegrelacAndNrorg($nrparcnegocio, $nrorg);
                if(!empty($interlocutor)){
                    $interlocutor->setIdativo('S');
                    $interlocutor->persist();
                }
            }else{
                $interlocutor = Relacionaparc::findOneByNrparcnegrelacAndNrorg($nrparcnegocio, $nrorg);
                if(!empty($interlocutor)){
                    $interlocutor->setIdativo('N');
                    $interlocutor->persist();
                }
            }

            if(in_array('VENDEDOR', $var)){
                $vendedor = Vendedor::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if($vendedor == null){
                    try{
                        $vendedor = new Vendedor();
                        $vendedor->setNrparcnegocio($nrparcnegocio);
                        $vendedor->setNmrazsocven($nome);
                        $vendedor->setNmfanven($nomereduzido);
                        $vendedor->setNrorg($nrorg);
                        $vendedor->setNrinsjurven($nrinscricaoparc);
                        $cdvendedor = Novocodigo::geraCodigo(Ambiente::getNrorg(), 'VENDEDOR', 1, 4);
                        $vendedor->setCdvendedor($cdvendedor);
                        $vendedor->setIdtpijurven($tipoinscricao[$cdtipoinscricao]);
                        $vendedor->setIdligextvend('N');
                        $vendedor->setIdgeracomisvend('S');
                        $vendedor->setIdcargovend('V');
                        $vendedor->setIdativo('S');
                        $vendedor->setDtinclusao(\Geral\Util\DateUtil::getDataAtual());
                        $vendedor->setCdoperinclusao(Ambiente::getCdOperador());
                        $vendedor->setNrorginclusao(Ambiente::getNrorg());
                        $vendedor->persist();
                    }catch(\Exception $e){
                        throw $e;
                    }
                }
                else{
                    $vendedor->setIdativo('S');
                    $vendedor->persist();
                }
            }else{
                $vendedor = Vendedor::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if($vendedor !== null){
                    $vendedor->setIdativo('N');
                    $vendedor->persist();
                }
            }

            if(in_array('PROSPECT', $var)){
                $prospect = Prospect::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if (!empty($prospect)){
                    $prospect->setIdativo('S');
                    $prospect->persist();
                }
            }
            else{
                $prospect = Prospect::findOneByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
                if (!empty($prospect)){
                    $prospect->setIdativo('N');
                    $prospect->persist();
                }
            }

            $connection->flush();
            $connection->commit();
            
            if (in_array('TRANSPORTADORA', $var))
                $inArray = true;
            else
                $inArray = false;
            
            self::rotinaTransportadora($inArray, $nrparcnegocio, $transportadora, $nrorg);

            $nrenderecoparc = EnderecoparcEntity::findByNrparcnegocioAndNrorg($nrparcnegocio, $nrorg);
            /** @var $enderecoparc EnderecoparcEntity*/
            foreach($nrenderecoparc as $enderecoparc) {
                Enderecoparc::rotinaParceriaEndereco($tipos, $nrparcnegocio, $enderecoparc->getNrenderecoparc(), $nrorg);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

     /**
     * Consulta se o objeto é novo
     *
     * @param Geral\Entities\Parceria $obj Objeto a ser consultado
     *
     * @return boolean
     *
     * @author gustavosilva
     *         17/05/2012
     */

    public static function isNewItemprebase(Parceria $obj)
    {
        if (self::isNull($obj->getNrparceria()) === true &&
            self::isNull($obj->getNrparcnegocio()) === false &&
            self::isNull($obj->getCdtipoparceiro()) === false){
                return true;
        } else {
            return false;
        }
    }

     /**
     * Consulta se o item é novo
     *
     * @param mixed $var        variavel a ser consultado
     *
     * @return boolean
     *
     * @author gustavosilva
     *         17/05/2012
     */
    public static function isNull(&$var){
        if ($var === null || $var === '') return true;
        return false;
    }

    public static function validaSave($nrinscricaoparc, $nrorg){
        try{
            $novo = ParcnegocioEntity::findOneByNrinscricaoparcAndNrorg($nrinscricaoparc, $nrorg);
            if ($novo instanceof ParcnegocioEntity) {
                throw new \Exception(tk_get_message_by_name('msg_parceiro_jcad'));
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function setarDtcompetencia()
    {
        $cdoperador = \Geral\Util\Ambiente::getCdoperador();
        $nrorg = \Geral\Util\Ambiente::getNrorg();
            
        $paramoper = \Geral\Entities\GpeParamoper::getObjectByWhere("CDOPERADOR = '$cdoperador'");
        $paramorg = \Folha\Entities\GpeParamorg::getObjectByWhere("NRORG = '$nrorg'");
        
        $true = false;

        if(is_object($paramoper))
        {
        
            $dtmescompetenc = $paramoper->getDtmescompetenc();
            if(!empty($dtmescompetenc)&&$paramoper->getIdtpdtcompete()=='OPERADOR')
            {   
                $dtcompoper = explode(' ',$dtmescompetenc);
                return $dtcompoper[0];
                $true = true;
            }
            if(is_object($paramorg)&&$true == false)
            {
                $dtcompetencia = $paramorg->getDtcompetencia();
                if(!empty($dtcompetencia) && $paramoper->getIdtpdtcompete() == 'ORGANIZACAO')
                {   
                    $dtcomporg = explode(' ',$dtcompetencia);
                    return $dtcomporg[0];
                    $true = true;
                }
            }
        }
        if(is_object($paramorg)&&$true==false)
        {   
            $dtcompetencia = $paramorg->getDtcompetencia();
            if(!empty($dtcompetencia) && $paramoper->getIdtpdtcompete() == 'ORGANIZACAO')
            {  
                $dtcomporg = explode(' ',$dtcompetencia);
                return $dtcomporg[0];
                $true = true;
            }
        }
        if($true == false)
        {
            return now();
        }
    }
}